local addonName, a = ...
local L = a.Localize
local s = SpellFlashAddon
local c = BittensGlobalTables.GetTable("BittensSpellFlashLibrary")

local GetPowerRegen = GetPowerRegen
local GetTime = GetTime
local UnitPowerMax = UnitPowerMax
local math = math
local select = select
local string = string
local tostring = tostring

a.Rotations = {}

local function t15RageBump(amount)
	if c.WearingSet(4, "ProtT15") 
		and s.Debuff(c.GetID("Demoralizing Shout")) then
		
		a.Rage = a.Rage + 1.5 * amount
	else
		a.Rage = a.Rage + amount
	end
end

local smashPending = 0

local monitorSmashPending = function(info)
	if c.InfoMatches(info, "Colossus Smash") then
		smashPending = GetTime()
		c.Debug("Event", "Colossus Smash Pending")
	end
end

local monitorSmashApplied = function(spellID)
	if c.IdMatches(spellID, "Colossus Smash") then
		smashPending = 0
		c.Debug("Event", "Colossus Smash Applied")
	end
end

function a.PreFlash()
	a.InExecute = s.HealthPercent() <= 20 or c.HasBuff("Sudden Death" ,false, false, true)
	a.Rage = c.GetPower(0)
	a.Enraged = c.HasBuff("Enrage", true)
	if c.IsQueued("Berserker Rage") then
		a.Rage = a.Rage + 10
		a.Enraged = true
	end
	if c.IsQueued("Charge") then
		t15RageBump(20)
	end		
	if c.IsQueued("Bloodthirst", "Mortal Strike") then
		a.Rage = a.Rage + 10
	elseif c.IsQueued("Battle Shout", "Commanding Shout") then
		a.Rage = a.Rage + 20
	elseif c.IsQueued("Shield Slam") then
		if s.Buff(c.GetID("Sword and Board"), "player") then
			t15RageBump(25)
		else
			t15RageBump(20)
		end
	elseif c.IsQueued("Revenge") then
		t15RageBump(15)
	end
	local max = UnitPowerMax("player")
	a.Rage = math.min(max, a.Rage)
	a.EmptyRage = max - a.Rage
	
	if c.IsCasting("Colossus Smash") or GetTime() - smashPending < .8 then
		a.Smash = 6
	else
		a.Smash = c.GetMyDebuffDuration("Colossus Smash", false, 6)
	end
	a.SmashCD = c.GetCooldown("Colossus Smash", false, 20)
	
	if c.IsCasting("Victory Rush", "Impending Victory") then
		a.VictoriousDuration = 0
	else
		a.VictoriousDuration = c.GetBuffDuration("Victorious")
	end
	
	a.Bloodbath = c.HasBuff("Bloodbath", false, false, true)
end

-------------------------------------------------------------------------- Arms
local tastePending = 0

a.Rotations.Arms = {
	Spec = 10,
	
	UsefulStats = { "Strength", "Melee Hit", "Crit", "Haste" },
	
	FlashInCombat = function()
		a.TasteStacks = c.GetBuffStack("Taste for Blood")
		if c.IsCasting("Mortal Strike") or GetTime() - tastePending < .8 then
			a.TasteStacks = a.TasteStacks + 2
		elseif c.IsCasting("Overpower") then
			a.TasteStacks = a.TasteStacks - 1
		end
		
		a.OverpowerIsFree = c.HasBuff("Sudden Execute", false, false, "Execute")
		a.CanOverpower = 
			a.TasteStacks > 0 and (a.Rage >= 10 or a.OverpowerIsFree)
		
		c.FlashAll(
			"Recklessness for Arms",
			"Avatar",
			"Bloodbath",
			"Berserker Rage for Arms",
			"Heroic Leap",
			"Impending Victory for Heals, Optional",
			"Victory Rush for Heals, Optional",
			"Enraged Regeneration",
			"Sunder Armor",
			"Pummel",
			"Disrupting Shout")
		if c.AoE then
			c.FlashAll("Cleave for Arms", "Sweeping Strikes")
			c.PriorityFlash(
				"Mortal Strike",
				"Bladestorm",
				"Shockwave",
				"Dragon Roar",
				"Thunder Clap",
				"Whirlwind for Arms",
				"Colossus Smash",
				"Overpower AoE",
				"Shout for Rage",
				"Storm Bolt",
				"Heroic Throw")
		else
			c.FlashAll("Heroic Strike for Arms")
			c.PriorityFlash(
				"Mortal Strike",
				"Storm Bolt for Arms",
				"Dragon Roar Prime for Arms",
				"Colossus Smash for Arms",
				"Execute for Arms",
				"Dragon Roar for Arms",
				"Slam Double Prime",
				"Overpower Prime",
				"Slam Prime",
				"Overpower",
				"Slam",
				"Shout for Rage",
				"Shockwave",
				"Impending Victory for Free",
				"Victory Rush",
				"Heroic Throw")
		end
	end,
	
	FlashAlways = function()
		c.FlashAll("Dps Stance", "Shout for Buff")
	end,
	
	CastSucceeded = function(info)
		if c.InfoMatches(info, "Mortal Strike") then
			tastePending = GetTime()
			c.Debug("Event", "Taste for Blood pending")
		else
			monitorSmashPending(info)
		end
	end,
	
	AuraApplied = function(spellID)
		if c.IdMatches(spellID, "Taste for Blood") then
			tastePending = 0
			c.Debug("Event", "Taste for Blood applied")
		else
			monitorSmashApplied(spellID)
		end
	end,
	
	ExtraDebugInfo = function()
		return string.format("b:%.1f r:%d e:%s e:%s t:%d o:%s f:%s s:%.1f", 
			c.GetBusyTime(), 
			a.Rage, 
			tostring(a.InExecute), 
			tostring(not not a.Enraged), 
			a.TasteStacks,
			tostring(a.CanOverpower),
			tostring(a.OverpowerIsFree),
			a.Smash)
	end,
}

-------------------------------------------------------------------------- Fury
a.CleaverPending = 0
a.CleaverDumpPending = 0

a.Rotations.Fury = {
	Spec = 2,
	
	UsefulStats = { "Strength", "Crit", "Mastery" },  -- "Haste", "Multistrike", "Versatility"},
	
	FlashInCombat = function()
		c.FlashAll(
		  "Berserker Rage for Fury",
			"Recklessness for Fury",
			"Avatar",
			"Bloodbath for Fury",
			"Impending Victory for Heals, Optional",
			"Victory Rush for Heals, Optional",
			"Enraged Regeneration",
			"Sunder Armor",
			"Pummel",
			"Disrupting Shout")
		if c.AoE then
			c.FlashAll("Bloodthirst")
			c.PriorityFlash(
	--			"Dragon Roar",
	--			"Shockwave",
	--			"Bladestorm",
	--			"Bloodthirst",
	--			"Raging Blow AoE",
	--			"Whirlwind",
	--			"Shout for Rage",
	--			"Storm Bolt",
				"Heroic Throw")
		else
			c.FlashAll("Bloodthirst")
			c.PriorityFlash(
			  "Wild Strike",
			  "Raging Blow",
				"Execute for Fury",
				"Bloodthirst",
				"Dragon Roar for Fury",
				"Storm Bolt",
				"Raging Blow")
		end
	end,
	
	FlashAlways = function()
		c.FlashAll("Dps Stance", "Shout for Buff")
	end,
	
	CastSucceeded = function(info)
		if c.InfoMatches(info, "Whirlwind") then
			a.CleaverPending = GetTime()
			c.Debug("Event", "Meat Cleaver Pending")
	--	elseif c.InfoMatches(info, "Raging Blow") then
	--		a.CleaverDumpPending = GetTime()
	--		c.Debug("Event", "Meat Cleaver Spend Pending")
		end
	end,
	
	AuraApplied = function(spellID)
		if c.IdMatches(spellID, "Meat Cleaver") then
			a.CleaverPending = 0
			c.Debug("Event", "Meat Cleaver Applied")
		end
	end,
	
	AuraRemoved = function(spellID)
		if c.IdMatches(spellID, "Meat Cleaver") then
			a.CleaverDumpPending = 0
			c.Debug("Event", "Meat Cleaver Spend Happened")
		end
	end,
	
	ExtraDebugInfo = function()
--	  local str = string.format("r:%d RB:%d e:%s b:%s t:%s", 
--      a.Rage, 
--      tostring(c.GetBuffDuration("Raging Blow")),
--      tostring(a.InExecute), 
--      tostring(c.HasBuff("Sudden Death", false, false, "Execute")),
--      tostring(c.HasTalent("Sudden Death", false, false, "Execute")))
--    print(str)
--		return string.format("r:%d e:%s e:%s", 
--			a.Rage, 
--			tostring(a.InExecute), 
--			tostring(not not a.Enraged))
--    return string.format("Stack:%d Rage:%d RB Duration:%d RB Casting:%s ER Casting:%s BloodSurge:%s",  -- Execute:%s",
    return string.format("Rage:%d BloodSurge:%s Execute:%s    ", 
      a.Rage, 
--      tostring(c.IsCasting("Raging Blow")),
--      tostring(c.IsCasting("Enrage")),
      tostring(c.HasBuff("Bloodsurge")),
      tostring(a.InExecute and c.HasBuff("Sudden Death", false, false, "Execute") and c.HasTalent("Sudden Death", false, false, "Execute")))
	end,
}

-------------------------------------------------------------------- Protection
local uncontrolledMitigationCooldowns = {
	"Spell Reflection",
}

a.RevengeReset = 0

a.Rotations.Protection = {
	Spec = 30,
	
	UsefulStats = { "Stamina", "Strength", "Dodge", "Parry", "Tanking Hit" },
	
	FlashInCombat = function()
		c.FlashMitigationBuffs(
			1,
			uncontrolledMitigationCooldowns,
			c.COMMON_TANKING_BUFFS,
			"No Mitigation if Victory Available",
			"Demoralizing Shout",
			"Enraged Regeneration for Prot",
			"Shield Wall under 3 min",
			"Last Stand",
			"Shield Wall 3+ min")
		c.FlashAll(
			"Shield Block",
			"Shield Barrier",
			"Berserker Rage for Prot",
			"Heroic Strike for Prot",
			"Cleave for Prot",
			"Taunt",
			"Sunder Armor",
			"Pummel",
			"Disrupting Shout")
		c.DelayPriorityFlash(
--			"Shockwave for Stun",
--			"Dragon's Roar for Stun",
			"Thunder Clap for Debuff",
			"Impending Victory for Heals",
			"Victory Rush for Heals",
			"Shield Slam",
			"Revenge",
			"Shout for Rage",
			"Delay for Prot Rage Generator",
			"Thunder Clap for Refresh",
			"Devestate for Debuff unless AoE",
			"Bladestorm",
			"Shockwave",
			"Dragon Roar",
			"Storm Bolt",
			"Devestate")
	end,
	
	FlashAlways = function()
		c.FlashAll("Defensive Stance", "Shout for Buff")
	end,
	
	Avoided = function(missType)
		if missType == "DODGE" or missType == "PARRY" then
			a.RevengeReset = GetTime()
			c.Debug("Event", "Revenge Reset Pending")
		end
	end,
	
	CastSucceeded = function(info)
		if c.InfoMatches(info, "Revenge") then
			a.RevengeReset = 0
			c.Debug("Event", "Revenge Cast")
		end
	end,
	
	ExtraDebugInfo = function()
		return string.format("r:%d e:%s e:%s, r:%.1f", 
			a.Rage, 
			tostring(a.InExecute), 
			tostring(not not a.Enraged),
			s.SpellCooldown(c.GetID("Revenge")))
	end,
}
